﻿using System;
using System.Windows;

namespace SilverlightApplication1
{
	public static class ScrollHelper
	{
		public static readonly DependencyProperty HorizontalOffsetProperty =
			DependencyProperty.RegisterAttached("HorizontalOffset", typeof(double), typeof(ScrollHelper), null);

		public static readonly DependencyProperty MaxTimeRulerExtentProperty =
			DependencyProperty.RegisterAttached("MaxTimeRulerExtent", typeof(double), typeof(ScrollHelper), new PropertyMetadata(double.PositiveInfinity, OnMaxTimeRulerExtentChanged));

		public static readonly DependencyProperty ExtentWidthProperty =
			DependencyProperty.RegisterAttached("ExtentWidth", typeof(double), typeof(ScrollHelper), new PropertyMetadata(OnExtentWidthChanged));

		public static readonly DependencyProperty ViewportWidthProperty =
			DependencyProperty.RegisterAttached("ViewportWidth", typeof(double), typeof(ScrollHelper), new PropertyMetadata(OnViewportWidthChanged));

		public static readonly DependencyProperty ContentWidthProperty =
			DependencyProperty.RegisterAttached("ContentWidth", typeof(double), typeof(ScrollHelper), null);

		public static readonly DependencyProperty HeadersLengthProperty =
			DependencyProperty.RegisterAttached("HeadersLength", typeof(double), typeof(ScrollHelper), new PropertyMetadata(OnHeadersLengthChanged));

		public static double GetHorizontalOffset(DependencyObject obj)
		{
			return (double)obj.GetValue(HorizontalOffsetProperty);
		}

		public static void SetHorizontalOffset(DependencyObject obj, double value)
		{
			obj.SetValue(HorizontalOffsetProperty, value);
		}

		public static double GetMaxTimeRulerExtent(DependencyObject obj)
		{
			return (double)obj.GetValue(MaxTimeRulerExtentProperty);
		}

		public static void SetMaxTimeRulerExtent(DependencyObject obj, double value)
		{
			obj.SetValue(MaxTimeRulerExtentProperty, value);
		}

		public static double GetExtentWidth(DependencyObject obj)
		{
			return (double)obj.GetValue(ExtentWidthProperty);
		}

		public static void SetExtentWidth(DependencyObject obj, double value)
		{
			obj.SetValue(ExtentWidthProperty, value);
		}

		public static double GetViewportWidth(DependencyObject obj)
		{
			return (double)obj.GetValue(ViewportWidthProperty);
		}

		public static void SetViewportWidth(DependencyObject obj, double value)
		{
			obj.SetValue(ViewportWidthProperty, value);
		}

		public static double GetContentWidth(DependencyObject obj)
		{
			return (double)obj.GetValue(ContentWidthProperty);
		}

		public static void SetContentWidth(DependencyObject obj, double value)
		{
			obj.SetValue(ContentWidthProperty, value);
		}

		public static double GetHeadersLength(DependencyObject obj)
		{
			return (double)obj.GetValue(HeadersLengthProperty);
		}

		public static void SetHeadersLength(DependencyObject obj, double value)
		{
			obj.SetValue(HeadersLengthProperty, value);
		}

		private static void OnMaxTimeRulerExtentChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			UpdateContentWidth(d);
		}

		private static void OnExtentWidthChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			UpdateContentWidth(d);
		}

		private static void OnViewportWidthChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			UpdateContentWidth(d);
		}

		private static void OnHeadersLengthChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			UpdateContentWidth(d);
		}

		private static void UpdateContentWidth(DependencyObject d)
		{
			SetContentWidth(d, Math.Max(0, Math.Min(GetMaxTimeRulerExtent(d), GetExtentWidth(d) + GetViewportWidth(d) - GetHeadersLength(d))));
		}
	}
}